#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'gjp'

"""File with Constants used in fiware-facts project.
"""

# HTTP CONSTANTS
CONTENT_HEADER = u'content-type'
JSON_TYPE = u'application/json'
REMOTE_ADDR = u'REMOTE_ADDR'
REMOTE_PORT = u'REMOTE_PORT'

# ORION CONSTANTS
CONTEXT_RESPONSES = u'contextResponses'
CONTEXT_ELEMENT = u'contextElement'
CONTEXT_ATTRIBUTES = u'attributes'
CONTEXT_ATTRIBUTES_NAME = u'name'
CONTEXT_ATTRIBUTES_VALUE = u'value'
